#include "ConfigBitsWizard.h"
#include <q3vbox.h>
#include <QMainWindow>
#include <QApplication>
#include <QToolBar>

//Added by qt3to4:
#include <Q3Frame>
#include <QPixmap>
#include <q3popupmenu.h>
#include <q3toolbar.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <q3hbox.h>
#include <qtooltip.h>
#include <qaction.h>


#include "pic.hh"
#include "ConfigBits.h"
#include "ConfigBitsWord.h"
#include "ConfigWordField.h"
#include "Project.h"
#include "ProjectWid.h"
#include "DraggableLabel.h"

#include <iostream>
#include <cstdio>
using namespace std ;


#include "configwL.xpm.h"
#include "configwH.xpm.h"
#include "configw.xpm.h"
#include "configwHL.xpm.h"

#include "pikdb.hh"

ConfigBitsWizard::ConfigBitsWizard ( ProjectWid *parent )
                : Q3MainWindow ( parent, "", Qt::WDestructiveClose ),
		deviceData ( 0 ), current_project ( 0 ), config ( 0 )
{

	// get project & pic device
	current_project = PikMain::mainWindow()->projectWid()->getProject() ;

	QString devname = current_project->getDevice() ;
	pikDB *db = pikDB::pikdb() ;
	deviceData = db->getData ( devname ) ;

        setCaption ( tr ( "Config bits editor / code generator (%1)" ).
                     arg ( deviceData->gpName ) ) ;


	Q3VBox *vb = new Q3VBox ( this ) ;
	vb->setSpacing ( 8 ) ;
	vb->setLineWidth ( 1 );
	vb->setMargin ( 8 ) ;
	setCentralWidget ( vb ) ;

	confView =    new  Q3ListView ( vb ) ;

        int firstcol = confView->addColumn ( tr ( "Config Words/Fields" ),200 ) ;
	confView->setColumnWidthMode ( firstcol,Q3ListView::Manual ) ;
	confView->setSorting ( -1 ) ; // no sort

	// projView->setColumnWidthMode (0, QListView::Maximum  );
        confView->addColumn ( tr ( "Role" ), 270 ) ;
        confView->addColumn ( tr ( "Current state" ), 400 ) ;

	confView->setRootIsDecorated ( true ) ;
	confView->setShowToolTips ( true ) ;
	confView->setAllColumnsShowFocus ( true ) ;

	// get list of config registers
	config = new ConfigBits ( deviceData ) ;

	// set config register from project
	slotReadConfig() ;


	Q3ValueVector<ConfigBitsWord>& thewords = config->theWords() ;

	for ( int i = thewords.size()-1 ; i >= 0 ; --i )
	{
		QString vals = "[" ;
		vals += thewords[i].binaryString() ;
		vals += "] [" ;
		vals += thewords[i].hexString() ;
		vals += "]" ;
		Q3ListViewItem * wordItem = new Q3ListViewItem ( confView, thewords[i].name(),"", vals ) ;
		wordItem ->setPixmap ( 2, QPixmap ( configwHL ) ) ;

		wordItem->setOpen ( true ) ;
		Q3ValueVector<ConfigWordField>& thefields = thewords[i].theFields() ;

		for ( int j =  0 ; j < ( int ) thefields.size() ; ++j )
		{
			QString name  = thefields[j].nameAndSize() ;
			QString desc = thefields[j].description() ;
			QString valuedesc = "" ;

			int index  = thefields[j].getValueIndex() ;
			int mini =  thefields[j].getMini();
			if ( index != -1 )
			{
                                valuedesc  = thefields[j].legalValueDesc(index) ;
                        }

			if ( name[0] !=  '0' && name[0] != '1' )
			{
				QString num ;
				num.sprintf ( "%d %d %d", i,j, index ) ;
				Q3ListViewItem *fieldItem =
				    new Q3ListViewItem ( wordItem, name, desc , valuedesc, num ) ;
				const char **ico ;

				if ( deviceData->core != 16 )
					ico = configw ;
				else
					ico =	( mini > 7 )  ? configwH : configwL ;

				fieldItem->setPixmap ( 0, QPixmap ( ico ) ) ;
			}

		}
		// projectTree->setPixmap ( 0, QPixmap ( ( const char** ) project_xpm ) ) ;
	}

	connect ( confView    , SIGNAL ( rightButtonPressed ( Q3ListViewItem *, const QPoint &, int ) ),
	          this, SLOT ( slotRButton ( Q3ListViewItem *, const QPoint &, int ) ) ) ;


        // make toobar 1 - this is not a classic toolbar,
        // because it contains draggable icons
        Q3ToolBar *bar1 = new Q3ToolBar ( this ) ;
	// to get icons ...

	// first draggable button
	DraggableLabel *dragCbutton = new DraggableLabel ( bar1 , "", config, 1 ) ;
        QToolTip::add ( dragCbutton, tr ( "Drag this icon to your C source code to get the current CONFIG bits definitions" ) ) ;

        QPixmap dragC(":/src/icons/22/pikdev_dragC.png");
        dragCbutton->setPixmap (dragC ) ;
	( new Q3HBox ( bar1 ) )->setFixedWidth ( 6 ) ;

	// second draggable button
	DraggableLabel *dragAbutton = new  DraggableLabel ( bar1 , "", config, 2 ) ;
        QToolTip::add ( dragAbutton, tr ( "Drag this icon to your ASM source code to get the current CONFIG bits definitions" ) ) ;
        QPixmap dragA(":/src/icons/22/pikdev_dragA.png");
        dragAbutton->setPixmap ( dragA ) ;
	( new Q3HBox ( bar1 ) )->setFixedWidth ( 6 ) ;


	// second toolbar for file generation and exit
        Q3ToolBar *bar2 =  new Q3ToolBar ( this ) ;

        QPixmap make_headers(":/src/icons/22/pikdev_make_headers.png");

        QAction *mh = new QAction (QIcon (  make_headers) , tr ("&Make header files" ),this )  ;
        mh->setShortcut(QKeySequence("Ctrl+H"));

	mh->addTo ( bar2 ) ;
	connect ( mh ,SIGNAL ( activated() ), this , SLOT ( slotMakeCode() ) ) ;

        QPixmap file_close(":/src/icons/22/pikdev_close.png");

        QAction *close = new QAction ( QIcon (  file_close ) ,tr ( "&Quit editor" ) ,this )  ;
        close->setShortcut(QKeySequence("Ctrl+Q"));

	close->addTo ( bar2 );
	connect ( close ,SIGNAL ( activated() ), this , SLOT ( slotClose() ) ) ;


	// menu
	Q3PopupMenu *menu = new Q3PopupMenu ;
        menuBar()->insertItem ( tr ( "Actions" ), menu ) ;
	mh->addTo ( menu ) ;
	close->addTo ( menu ) ;

        statusBar()->message ( tr ( "  Current device is %1  " ).arg ( deviceData->gpName ) , 10000 ) ;
        setWindowIcon(QPixmap(":/src/icons/16/pikdev_icon.png") ) ;
        show();
}


ConfigBitsWizard::~ConfigBitsWizard()
{
	delete config ;
}

void ConfigBitsWizard::slotRButton ( Q3ListViewItem *li, const QPoint &p, int )
{

	if ( li != 0 )
	{
		if ( li->text ( 3 )  != "" )
		{
			// decode config word and field numbers
			int wn, fn , value_index ;
			if ( sscanf ( li->text ( 3 ) .latin1(), "%d %d %d", &wn,&fn, &value_index ) != 3 ) return ;

			Q3ValueVector<ConfigBitsWord>& thewords = config->theWords() ;
			Q3ValueVector<ConfigWordField>& thefields = thewords[wn].theFields() ;

                        Q3PopupMenu *pop = makePopup ( tr ( "Select a configuration" ), this ) ;

			for ( int i=0 ; i < thefields[fn].legalValuesCount() ; ++i )
				pop->insertItem ( thefields[fn].legalValueDesc ( i ) ,i ) ;

			int menu_index = pop->exec ( p )  ;
			if ( menu_index  >= 0 )
			{
				thefields[fn].setValue ( menu_index ) ;
				li->setText ( 2, thefields[fn].legalValueDesc ( menu_index ) ) ;
				QString s ;
				s.sprintf ( "%d %d %d", wn,fn,menu_index ) ;
				li->setText ( 3, s ) ;

				// now update config object
				thefields[fn].setValue ( menu_index ) ;

				// and save new config to project file
				slotWriteConfig() ;

				// update config word display
				Q3ListViewItem *parent = li->parent() ;
				QString vals = "[" ;
				vals += thewords[wn].binaryString() ;
				vals += "] [" ;
				vals += thewords[wn].hexString() ;
				vals += "]" ;
				parent->setText ( 2, vals ) ;
			}
		}
	}
}

/** Make an empty popup menu with "title" header */
Q3PopupMenu *ConfigBitsWizard::makePopup ( const QString& txt, QWidget *parent )
{
	Q3PopupMenu *pop = new   Q3PopupMenu ( parent ) ;

        pop->addAction ( QIcon(":/src/icons/16/check.png"), txt )->setEnabled(false);
        pop->addSeparator() ;

	return pop ;

}

void ConfigBitsWizard::slotWriteConfig()
{
	current_project->setConfigWords ( config->getRawValues() ) ;
}

void ConfigBitsWizard::slotReadConfig()
{
	// read raw config bits values from project
	Q3ValueVector<int> values = current_project->getConfigWords() ;

	// set the ConfigBits object accordingly
	const char *message ;
	if ( ( message = config->setRawValues ( values ) ) != 0 )
	{
                statusBar()->message (QString(message), 10000) ;
	}
}


/*!
    \fn ConfigBitsWizard::makeCode()
 */
void ConfigBitsWizard::slotMakeCode()
{

	QString dir = current_project ->projectPath() ;
	config->writeCCode ( dir ) ;
	config->writeASMCode ( dir ) ;
	statusBar()->message (
            tr ( " [config_bits.h] and [config_bits.inc] created  in %1 " )
	    .arg ( dir ) , 10000 ) ;
}


/*!
    \fn ConfigBitsWizard::slotClose()
 */
void ConfigBitsWizard::slotClose()
{
	emit closed() ;
	close ( true ) ;
}


bool ConfigBitsWizard::queryClose()
{
	emit closed() ;
	return true ;
}

void  ConfigBitsWizard::closeEvent(QCloseEvent *event)
{
    emit closed() ;
    event->accept();
}
