#include "ConfigBitsWord.h"
#include "ConfigWordField.h"

ConfigBitsWord::ConfigBitsWord ( const QString& name, int addr , const QString& desc )
{
	const char* p = desc.latin1() ;
	ConfigWordField f ;

	word_addr = addr ;
	word_name = name ;

	for ( ; f.parse ( p ) ; )
	{
		fields.push_back ( f ) ;
	}

}

void ConfigBitsWord::dump ( ostream& os )
{
        os << word_name.toStdString() ;
        os << hex << " ADDR=" << word_addr << endl ;

        for (  int i=0 ; i < fields.size() ; ++i )
	{
		fields[i].dump ( os ) ;
		os << endl ;
	}
}

ConfigBitsWord::~ConfigBitsWord()
{
}


int ConfigBitsWord::value() const
{
	int v = 0 ;
        for (  int i=0 ; i < fields.size() ; ++i )
	{
		v |= fields[i].getValue() ;
	}
	return v ;
}


/*!
    \fn ConfigBitsWord::setValue(int)
 */
void ConfigBitsWord::setValue ( int v )
{
        for (  int i = 0 ; i < fields.size() ; ++i )
	{
		fields[i].makeFieldFromWord ( v ) ;
	}
}


/*!
    \fn ConfigBitsWord::humanReadable()
 */
QString ConfigBitsWord::humanReadable ( const QString& newline )
{
	QString r = name() ;
	r += " at 0x" ;
	r += QString::number ( addr(), 16 ) ;
	r += " value = 0x" ;
	r += QString::number ( value(), 16 ) ;
	r += " - 0b" ;
	r += binaryString() ;
	r += newline ;
	r += "---------------------" ;
	r += newline ;

	for ( int i = 0 ; i < ( int ) theFields().size() ; ++i )
	{
		r += ( theFields() [i] ).humanReadable() ;
		r += newline ;
	}
	return r ;
}


/*!
    \fn ConfigBitsWord::mask()
 */
int ConfigBitsWord::mask()
{
	int v = 0 ;
        for (  int i=0 ; i < fields.size() ; ++i )
	{
		v |= fields[i].mask() ;
	}
	return v ;
}


/*!
    \fn ConfigBitsWord::binaryString()
 */
QString  ConfigBitsWord::binaryString()
{
	QString r, sep ;
	

        for (  int i= 0 ; i < fields.size() ; ++i)
	{
		r += sep ; sep = "_" ;
		r +=  fields[i].binaryString() ;
	}
	return r ;
   
}
/*!
    \fn ConfigBitsWord::hexString()
 */
QString  ConfigBitsWord::hexString()
{
	QString r ;
	r.sprintf("0x%04X", value()) ;
	return r ;
}

QString  ConfigBitsWord::symbolicString()
{
        QString r ;
        for( int i= 0 ; i < fields.size() ; ++i)
        {
            QString f = fields[i].symbolicString() ;
            if( f != "")
            {
                if(r != "") r += "," ;
                r += f ;
            }
        }
        return r ;
}
