/***************************************************************************
                          GlobalConfigWidget.cpp  -  description
                             -------------------
    begin                : jeu fv 2 2006
    copyright            : (C) 2006 by Gibaud Alain
    email                : alain.gibaud@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "GlobalConfigWidget.h"
#include "PikMain.h"
#include "Config.h"
#include "XQPushButton.h"

#include <QComboBox>
#include <Q3GroupBox>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QToolTip>
#include <QMessageBox>
#include <QFileDialog>
#include <QPixmap>
#include <Q3Frame>

#include <iostream>
using namespace std ;


GlobalConfigWidget::GlobalConfigWidget(PikMain *pikmain) :
        ConfigWidget(0) , main_widget(pikmain)
{
    Q3VBox  *central = new Q3VBox(this) ;
    central->setSpacing(8) ;
    central->setLineWidth( 2 );
    central->setMargin(8) ;

    Q3GroupBox *programmersbox = new Q3GroupBox(1, Qt::Horizontal, tr("Programmers") ,  central ) ;
    programmersbox->setInsideMargin(14) ;
    programmers = new QComboBox(programmersbox) ;

    // get current programmer shortname
    QString shortname ;
    Config::getCurrentProgrammer(shortname) ;

    // get programmers names
    Q3ValueVector<ProgrammerBase *>& progs = main_widget->programmersList() ;
    int cp = 0 ;
    QString path ;
    for(int i = 0 ; i < (int)progs.size() ; ++i)
    {
        programmers->insertItem(progs[i]->longName()) ;
        if(progs[i]->shortName() == shortname)
            cp = i ;
    }
    // display current programmer
    programmers->setCurrentItem(cp) ;

    programmersbox->setFixedHeight(programmersbox->sizeHint().height());

    Q3GroupBox *toolsbox = new Q3GroupBox(3, Qt::Horizontal, tr("External tools") ,  central ) ;
    toolsbox->setInsideMargin(14) ;

    QStringList toolnames ;
    Config::getToolNames(toolnames) ;
    QStringList::iterator t ;
    int nb ;


    QIcon iconset(":src/icons/22/pikdev_openfile.png") ;

    for(t = toolnames.begin(), nb = 0 ; t != toolnames.end() ; ++t, ++nb)
    {
        QString path, info , URL ;
        bool mand ;
        Config::getToolEntry(*t, path, info, URL, mand) ;

        QLabel *lab = new QLabel(*t, toolsbox) ;
        tool_names.push_back(lab) ;
        QLineEdit *p = new QLineEdit( path , toolsbox) ;
        tool_paths.push_back(p) ;

        QString tttext = info ;
        tttext += tr("\nDownload:  ") ;
        tttext += URL ;
        QToolTip::add(p, tttext) ;

        XQPushButton *b = new XQPushButton(QString::null, toolsbox, nb) ;
        connect(b, SIGNAL(clicked(int)), this , SLOT(chooseExecutable( int ))) ;
        b->setIconSet(iconset) ;
    }

    new Q3VBox(central) ;
}
GlobalConfigWidget::~GlobalConfigWidget()
{}
/** short name of configurator */
QString GlobalConfigWidget::configName()
{
    return tr("Global") ;
}
/** title string for this configurator */
QString GlobalConfigWidget::configTitle()
{
    return tr("Global settings");
}
/** return icon of this configurator  */
QPixmap GlobalConfigWidget::icon()
{
    return QPixmap(":/src/icons/32/pikdev_config_general.png") ;
}


/** apply configured data (ie: save them to config file) */
void GlobalConfigWidget::doApply()
{
    QString longprogname = programmers->currentText() ;

    Q3ValueVector<ProgrammerBase *> &progs =  main_widget->programmersList() ;

    // update paths for each tool
    for(int  i = tool_names.size() -1  ; i >= 0 ; --i)
    {
        QString path, URL, info, toolname = tool_names[i]->text()   ;
        bool mand ;
        Config::getToolEntry(toolname, path, info, URL, mand) ;
        if( mand &&  tool_paths[i]->text() == "")
        {

            QMessageBox::warning ( this, tr("Installation needed"),
            tr("Pikdev needs %1 to work properly.\nPlease install this tool (see %2)").arg(toolname).arg(URL) ) ;
        }
        else
	{
		QString exename = tool_paths[i]->text() ;
		
		if( QFile::exists(exename) )
                        Config::setToolPath(toolname, exename ) ;
		else if( exename != "")
		{

                    QMessageBox::warning ( this, tr("Incorrect path"),
                    tr("External tool %1 doesn't exist.\nPlease fix this path.").arg(exename) ) ;
		}
		
	}
    }

    // get programmer long name from combobox
    // and find corresponding short name
    QString programmername  ;
    for( int i =  progs.size()-1 ; i >= 0 ; --i )
    {
        if( longprogname == progs[i]->longName())
        {
            programmername  = progs[i]->shortName();
            break ;
        }
    }

    // is programmer installed & configured ?
    QString path ;
    Config::getToolPath(programmername, path ) ;
    if( programmername != "Builtin" && path == "")
    {
        QMessageBox::warning ( this, tr("Installation needed"),
        tr("Programming tool not found.\nPlease install the application %1").arg(programmername)  ) ;
        Config::setCurrentProgrammer( "Builtin" ) ;
    }
    else 
    {
        Config::setCurrentProgrammer(  programmername  ) ;

    }
    main_widget->updateProgrammer() ;

}


/*!
    \fn GlobalConfigWidget::chooseExecutable(int i)
 */
void GlobalConfigWidget::chooseExecutable(int i)
{
    QString fname = QFileDialog::getOpenFileName(this, "/usr" , tr("Executable (*)")) ;
    tool_paths[i]->setText(fname) ;
}
