/***************************************************************************
                          NewProject.cpp  -  description
                             -------------------
    begin                : ven nov 28 2003
    copyright            : (C) 2003 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "NewProject.h"
#include "FNU.h"
#include "PikMain.h"

#include <q3vbox.h>
#include <q3hbox.h>
#include <q3groupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlineedit.h>
#include <q3textedit.h>
#include <qpushbutton.h>
#include <q3grid.h>
#include <qicon.h>
#include <qdir.h>
#include <q3url.h>
#include <QMessageBox>
#include <q3vgroupbox.h>
#include <QComboBox>
#include <QCheckBox>
#include <QFileDialog>
//Added by qt3to4:
#include <Q3Frame>


#include <iostream>
using namespace std ;


NewProject::NewProject( ) : Q3MainWindow(0,"new project widget", Qt::WDestructiveClose )
{
    QIcon iconset (":/src/icons/22/pikdev_openfile.png") ;
    
    setCaption(tr("Project creation")) ;
    
    Q3VBox *vb = new Q3VBox( this) ;
    vb->setSpacing(8) ;
    vb->setLineWidth( 2 );
    vb->setMargin(8) ;
    setCentralWidget(vb) ;

    Q3VGroupBox *vgb =   new Q3VGroupBox(tr("Project informations"), vb) ;
    vgb->setInsideSpacing(8) ;
    
    Q3Grid *group = new Q3Grid(3,vgb) ;
    group->setSpacing(10) ;

    new QLabel (tr("Name") , group) ;
    project_name = new  QLineEdit(group) ;
    connect(project_name, SIGNAL(textChanged(const QString &)),
    	this, SLOT(slotNameChanged(const QString &))) ;
    new  Q3HBox(group) ;
    
    new QLabel ( tr("Directory") , group) ;
    project_dir = new  QLineEdit( group) ;
    QPushButton *bdir = new  QPushButton(iconset,"", group) ;
    connect(bdir,SIGNAL(clicked()), this, SLOT( slotSelectDir())) ;

    new QLabel ( tr("Type") , group) ;
    project_type = new  QComboBox(false, group) ;
    project_type->insertItem(tr("Asm")) ;
    project_type->insertItem(tr("Cpik for p18xxx")) ;
    new  Q3HBox(group) ;


    
    new QLabel ( tr("Configure now") , group) ;
    config_now = new QCheckBox( group) ;
    config_now->setChecked(true) ;
    new  Q3HBox(group) ;

    Q3HBox *boxbutton = new Q3HBox(vb) ;
    boxbutton->setSpacing(10) ;
    new Q3HBox(boxbutton) ;
    
    QPushButton * create = new QPushButton(tr("Create"), boxbutton) ;
    connect(create, SIGNAL(clicked()), this, SLOT(slotCreate())) ;
    
    QPushButton * cancel = new QPushButton(tr("Cancel"), boxbutton) ;
    connect(cancel, SIGNAL(clicked()), this, SLOT(slotCancel())) ;
    
    setMinimumWidth(500) ;
    show() ;
}
NewProject::~NewProject()
{
}
/** This slot must be activated from the browse button and update
the project_dir lineedit. */
void NewProject::slotSelectDir()
{
    QString dir =
      QFileDialog::getExistingDirectory ( this,   tr("Select project directory"),  "" ,
                                          QFileDialog::ShowDirsOnly ) ;


    if( !dir.isEmpty())
    {
        project_dir->setText(dir) ;
        slotNameChanged(project_name->text()) ;
    }
}

/** No descriptions */
void NewProject::slotCancel()
{
  close(true) ;
}
/** No descriptions */
void NewProject::slotNameChanged(const QString &name)
{
	QString dir(   project_dir->text()  ) ;

	if( dir == "") dir=QDir::homeDirPath() + "/" ;

	project_dir->setText(FNU::path(dir) + "/" + name) ;
}
	
/** No descriptions */
void NewProject::slotCreate()
{
  if( createProjectEntry() && config_now->isChecked() )
  {
    // project entry created, do more configuration.
      if(PikMain::mainWindow())
          PikMain::mainWindow()->projectWid()->slotEditProject() ;
  }
  slotCancel() ;
}

/** No descriptions */
bool NewProject::createProjectEntry()
{
  if( project_name->text() == "" || project_dir->text() == "" )
  {
        QMessageBox::warning(this,tr("Sorry"),
                             tr("<b>Directory</b> and <b>Project name</b>"
                             "  must be provided"),
                             QMessageBox::Ok, Qt::NoButton, Qt::NoButton) ;


       return false;
  }

  QString type ;
  int t =  project_type->currentItem() ;

  if(t == 0) type = "asm" ;
  else if( t == 1 ) type = "C" ;
  else return false;

  emit projectData( project_name->text() , project_dir->text(), type )  ;

  return true ;

}
