#include "PrgWid.h"
#include "MemoryDisplayer.h"
#include "BinWordsEditor.h"
#include "q3vbox.h"
#include <QPushButton>
#include <QMessageBox>


#include <QLabel>
#include <Q3Frame>
#include <QApplication>
#include <QDesktopWidget>
#include <QMenu>
#include <QMenuBar>
#include <QToolBar>
#include <QAction>
#include <QStatusBar>
#include <QProgressBar>
#include <QFileDialog>
#include <QApplication>



#include <q3groupbox.h>
#include <q3hgroupbox.h>
#include <q3filedialog.h>
#include <QRegExp>

#include <QEvent>
#include <QTimer>
#include <QToolTip>
#include <QComboBox>
#include <QStringList>

#include <sstream>
#include <iostream>
#include <assert.h>

using namespace std ;


#include "ConfigBits.h"

// ------------------------------------------------------

PrgWid::PrgWid ( ProgrammerBase *programmer_, const QString& fn, QWidget *parent ) :
                QMainWindow ( parent),
		fileName ( fn ), curpic ( programmer_->device() ),
		programmer ( programmer_ )
{

	FragBuffer *mem ;
	int mem1, mem2, ee1, ee2, id1,id2, cf1,cf2, osccal_backup1, osccal_backup2 ;
	QString bits ;
	unsigned int memdisplay_h = 10 ; // number of lines for programm memory displayer


        setCaption(tr("Programmer window")) ;

	// for low resolution devices (ie: netbooks)
	QFont f = font() ;
        if ( QApplication::desktop()->availableGeometry ( -1 ).height() <=600 )
		f.setPointSize ( 6 ) ;
	else
		f.setPointSize ( 10 ) ;
	this->setFont ( f ) ;

	setCaption ( programmer->longName() ) ;

	// Fonctions "file"
	// definition des actions 
        actions.addAction( "open-hex" , ":/src/icons/22/document-open.png", tr("&Open"), QKeySequence(tr("Ctrl+O")), this) ;
        actions.connect("open-hex", this, SLOT ( loadHex() ) ) ;

        actions.addAction( "reload-hex" , ":/src/icons/22/document-revert.png", tr("Reloa&d"), QKeySequence(tr("Ctrl+D")), this) ;
        actions.connect("reload-hex", this, SLOT ( loadHexAs() ) ) ;

        actions.addAction( "save-hex" , ":/src/icons/22/document-save.png", tr("&Save"), QKeySequence(tr("Ctrl+S")), this) ;
        actions.connect("save-hex", this, SLOT ( saveHex() ) ) ;

        actions.addAction( "saveas-hex" , ":/src/icons/22/document-save-as.png", tr("&Save As"), QKeySequence(tr("Ctrl+A")), this) ;
        actions.connect("saveas-hex", this, SLOT ( saveHexAs() ) ) ;

        actions.addAction( "quit-prog" , ":/src/icons/22/pikdev_close.png", tr("&Quit"), QKeySequence(tr("Ctrl+Q")), this) ;
        actions.connect("quit-prog", this, SLOT ( quit() ) ) ;


	//
        // Les actions "chip"
	//
        actions.addAction( "readchip" , ":/src/icons/22/readchip.png", tr("&Read Pic"), QKeySequence(tr("Ctrl+R")), this) ;
        actions.connect("readchip", this, SLOT ( readPic() ) ) ;

        actions.addAction( "erasechip" , ":/src/icons/22/erasechip.png", tr("&Erase Pic"), QKeySequence(tr("Ctrl+E")), this) ;
        actions.connect("erasechip", this, SLOT ( erasePic() ) ) ;

        actions.addAction( "burnchip" , ":/src/icons/22/pikdev_burnchip.png", tr("&Program Pic"), QKeySequence( Qt::Key_F), this) ;
        actions.connect("burnchip", this, SLOT ( programPic() ) ) ;

        actions.addAction( "verifychip" , ":/src/icons/22/verifychip.png", tr("&Verify Pic"), QKeySequence( tr("Ctrl+V") ), this) ;
        actions.connect("verifychip", this, SLOT ( verifyPic() ) ) ;


        //
        // create menus, and plug actions
        //
        QMenu *file = new QMenu("&File",this) ;
        menuBar()->addMenu(file) ;
        file->addAction(actions.act("open-hex")) ;
        file->addAction(actions.act("reload-hex")) ;
        file->addAction(actions.act("save-hex")) ;
        file->addAction(actions.act("saveas-hex")) ;
        file->addAction(actions.act("quit-prog")) ;
        QToolBar *tfile = new QToolBar("&File",this) ;
        addToolBar(tfile);
        tfile->addAction(actions.act("open-hex")) ;
        tfile->addAction(actions.act("reload-hex")) ;
        tfile->addAction(actions.act("save-hex")) ;
        tfile->addAction(actions.act("saveas-hex")) ;
        tfile->addAction(actions.act("quit-prog")) ;




        QMenu *picops = new QMenu("Pic Operations",this) ;
        menuBar()->addMenu(picops) ;
        picops->addAction(actions.act("readchip")) ;
        picops->addAction(actions.act("erasechip")) ;
        picops->addAction(actions.act("burnchip")) ;
        picops->addAction(actions.act("verifychip")) ;

        QToolBar *tpicops = new QToolBar("Pic Operations",this) ;
        addToolBar(tpicops) ;
        tpicops->addAction(actions.act("readchip")) ;
        tpicops->addAction(actions.act("erasechip")) ;
        tpicops->addAction(actions.act("burnchip")) ;
        tpicops->addAction(actions.act("verifychip")) ;



        QMenu *extra = new QMenu("&Extras",this) ;
        menuBar()->addMenu(extra) ;
        connect(extra, SIGNAL(aboutToShow()),
                this, SLOT ( slotExtraMenuManagement() ) );


	// ------------------------------------------------------------------


	// La boite horizontale la plus externe

	Q3HBox *h1 = new Q3HBox ( this ) ;

        h1->setSpacing ( 8 ) ;
	h1->setLineWidth ( 2 );
	h1->setMargin ( 8 ) ;

	Q3VBox *v1 = new Q3VBox ( h1 ) ;
	v1->setSpacing ( 8 ) ;

	//   display current settings
	Q3HBox *settingh = new  Q3HBox ( v1 ) ;
        settingh->setSpacing ( 3 ) ;
        Q3HGroupBox *settingbox = new Q3HGroupBox ( tr ( "Current settings" ) , settingh ) ;
        settingbox->setInsideMargin(2);

	QLabel *pname = new QLabel ( curpic->name(), settingbox ) ;
	pname->setPaletteForegroundColor ( QColor ( 255,0,0 ) ) ;
	settingbox->setMaximumHeight ( settingbox->sizeHint().height() +8 ) ;

	settingh-> setStretchFactor ( settingbox , 10 ) ;

	fileNameLabel = new QLineEdit ( settingbox ) ;
	fileNameLabel->setReadOnly ( true ) ;

	setFileName ( fileName ) ;


	// init hook for progress bar management
	curpic->setProgressListener ( this ) ;

	curpic->setProgOptionNumber ( -1 ) ;
	// display programming options, if any
	if ( *curpic->progOptions() )
	{
		memdisplay_h -= 3 ; // save space
                Q3HGroupBox *optbox = new Q3HGroupBox ( tr ( "Programming options" ) , v1 ) ;
		QComboBox *progopt = new QComboBox ( optbox ) ;
		progopt->insertStringList ( parseProgOptions ( curpic->progOptions() ) ) ;
		slotProgOptionSelected ( 0 ) ;
		connect ( progopt, SIGNAL ( activated ( int ) ),  this, SLOT ( slotProgOptionSelected ( int ) ) ) ;
	}

	curpic->getBuffers ( &mem,  mem1, mem2, ee1,  ee2,id1, id2, cf1, cf2,
	                     osccal_backup1, osccal_backup2 ) ;

        bits = QString ( tr ( "(%1 bits words)" ) ).arg ( curpic->wwidth() ) ;

	MemoryDisplayer *md1 = new MemoryDisplayer ( v1,*mem,
	        mem1, mem2-mem1+1  , memdisplay_h, 8U,
                tr ( "Program memory " ) + bits ,
	        curpic->wmask(), curpic->byteAdresses(),
	        curpic->SPLocs() );
	connect ( this,SIGNAL ( bufferModified() ), md1, SLOT ( updateDisplay() ) ) ;

	if ( ee1 != -1 )
	{
                QString mess = tr ( "Data memory (8 bits words)" )  ;
		unsigned int mask = 0xFF ;
		if ( curpic->deviceCore() == 16 )
		{
                        mess = tr ( "Data memory (packed 8 bits words)" )  ;
			mask = 0xFFFF ;
		}
		MemoryDisplayer *md2 = new MemoryDisplayer ( v1,*mem,
		        ee1, ee2-ee1+1  , 6U, 8U, mess ,mask, curpic->byteAdresses(),
		        curpic->SPLocs() );
		connect ( this,SIGNAL ( bufferModified() ), md2, SLOT ( updateDisplay() ) ) ;
	}

	Q3HBox *h2 = new Q3HBox ( v1 ) ;
	if ( id1 != -1 )
	{
		unsigned int nbid = id2-id1+1   ;
		unsigned int nbcol =  nbid < 8 ? nbid : 8 ;
		unsigned int nbline = ( ( nbid / nbcol ) > 1 ) ? 2 : 1 ;

                QString mess =   tr ( "ID data " ) +bits  ;
		if ( curpic->deviceCore() == 16 )
                        mess = tr ( "ID data (packed 8 bits words)" )  ;
		MemoryDisplayer *md3 = new MemoryDisplayer ( h2,*mem,
		        id1, nbid  , nbline , nbcol , mess ,curpic->wmask(),
		        curpic->byteAdresses(),
		        curpic->SPLocs() );
		h2->setStretchFactor ( md3,0 ) ;
		connect ( this,SIGNAL ( bufferModified() ), md3, SLOT ( updateDisplay() ) ) ;
	}
	//  this is a filler (which absorb vertical space) for devices with no EEPROM memory
	new Q3VBox ( v1 ) ;

	// filler a droite de ID data
	Q3HBox *pad1 = new Q3HBox ( h2 ) ;
	pad1->setSpacing ( 0 ) ;
	h2->setStretchFactor ( pad1,5 ) ;
	h2->setSpacing ( 8 ) ;

	// ----------------------
	Q3VBox *v2 = new Q3VBox ( h1 ) ;
        v2->setSpacing (3 ) ;

	BinWordsEditor *confuses = new  BinWordsEditor ( *mem, cf1, cf2,
                curpic->config_bits() , tr ( "Config Fuses" ), curpic->byteAdresses(), v2 ) ;
	connect ( this,SIGNAL ( bufferModified() ), confuses, SIGNAL ( wordsChanged() ) ) ;

	v2->setStretchFactor ( confuses, 2 ) ;

	// display osccal backup
	if ( osccal_backup1 != -1 )
	{
                QString mess =   tr ( "Calibration" ) ;

		int nbo =  osccal_backup2 -  osccal_backup1 + 1 ;
		MemoryDisplayer *md4 = new MemoryDisplayer ( v2,*mem,
		        osccal_backup1 , nbo  , nbo , 1 , mess ,curpic->wmask(),
		        curpic->byteAdresses(),
		        curpic->SPLocs() );
		v2->setStretchFactor ( md4,0 ) ;
		connect ( this,SIGNAL ( bufferModified() ), md4, SLOT ( updateDisplay() ) ) ;
	}

	setCentralWidget ( h1 ) ;

	statBar = statusBar()  ;
	Q3HBox *statBox = new   Q3HBox ( this ) ;
	statBar->addWidget ( statBox ) ;

        progRunning  = new Indicator(statBox) ;
	statusLabel =   new QLabel ( statBox ) ;
        progress = new QProgressBar(  statBox ) ;
        progress->setRange(0,100);

	Q3HBox *detectBox =  new   Q3HBox ( this ) ;
	statBar->addWidget ( detectBox,10 ) ;

        new QLabel ( tr ( "  Device: " ), detectBox ) ;
	detected  = new QLineEdit ( detectBox ) ;
	detected->setReadOnly ( true ) ;
	detectBox->setStretchFactor ( detected,1 ) ;
        new QLabel ( tr ( " Rev: " ), detectBox ) ;
	revdetected = new QLineEdit ( detectBox ) ;
	revdetected->setMaximumWidth ( 24 ) ;
	revdetected->setReadOnly ( true ) ;

	new QLabel ( "  ", detectBox ) ;   // spacer

        QPushButton *detectButton =   new QPushButton ( tr ( "Detect now" ),detectBox )     ;

	detectButton->setFont ( f ) ;
	detectButton->setMaximumHeight ( 18 ) ;
	detectButton ->updateGeometry() ;

	connect ( detectButton, SIGNAL ( clicked() ), this , SLOT ( slotDetectDeviceType() ) ) ;

        setStatus ( false,QString ( tr ( "Ok" ) ) ) ;

        setWindowIcon(QPixmap(":/src/icons/16/pikdev_icon.png") ) ;


}
PrgWid::~PrgWid()
{
	curpic->setProgressListener ( 0 ) ;
}

void PrgWid::go()
{

        setMaximumHeight( sizeHint().height()  ) ;
	show() ;
}

void PrgWid::quit()
{
	close ( true ) ;
}

void PrgWid::saveHex()
{
	const char *p ;
	p = curpic->writeHexFile ( ( const char * ) fileName , "inhx32" ) ;
	if ( p != 0 )
	{
                QMessageBox::critical ( this,tr ( "Sorry" ),p,tr ( "Ok" ) ) ;
	}
	else
		emit fileUpdated ( fileName ) ;
}


void PrgWid::saveHexAs()
{
	QString fname
              = QFileDialog::getOpenFileName(this, "Select a file", ".", "Intel hex files (*.hex *.HEX)" ) ;

	if ( !fname.isEmpty() )
	{

		const char *p ;
		p = curpic->writeHexFile ( ( const char * ) fname , "inhx32" ) ;
		if ( p != 0 )
		{
                        QMessageBox::critical ( this,tr ( "Sorry" ),p,tr ( "Ok" ) ) ;
		}
		else
		{
			emit fileUpdated ( fname ) ;
			setFileName ( fname ) ;
		}
	}
}



// reload
void PrgWid::loadHex()
{
	const char *p ;
	p = curpic->loadHexFile ( ( const char * ) fileName ) ;
	//curpic->display_mem(cout) ;
	if ( p != 0 )
	{
                QMessageBox::critical ( this,tr ( "Sorry" ),p,tr ( "Ok" ) ) ;
	}
	else
		emit bufferModified() ;
}


void PrgWid::loadHexAs()
{
	QString fname
        = QFileDialog::getOpenFileName(this, "Select a file", ".", "Intel hex files (*.hex *.HEX)" ) ;

	if ( !fname.isEmpty() )
	{
		const char *p ;
		p = curpic->loadHexFile ( ( const char * ) fname ) ;
		if ( p != 0 )
		{
                        QMessageBox::critical ( this,tr ( "Sorry" ),p,tr ( "Ok" ) ) ;
		}
		else
		{
			emit bufferModified() ;
			setFileName ( fname ) ;
		}
	}
}


void PrgWid::readPic()
{

	if ( ! verifyDeviceType() )
		return ;

	QString qtxt ;
	emit messageGenerated ( qtxt ) ;
        setStatus ( true, tr ( "Reading ..." ) ) ;
        QApplication::processEvents ( QEventLoop::ExcludeUserInputEvents , 200 ) ;
	curpic->read_pic() ;
	emit bufferModified() ;
        setStatus ( false, tr ( "Ok" ) );
}


void  PrgWid::erasePic()
{
	if ( ! verifyDeviceType() )
		return ;

	QString qtxt ;

	emit messageGenerated ( qtxt ) ;
        setStatus ( true,tr ( "Erasing ..." ) )   ;
        QApplication::processEvents ( QEventLoop::ExcludeUserInputEvents , 200 ) ;
	int i = curpic->erase_pic() ;
	if ( i != 0 )
		setStatus ( false,curpic->error_msg ( i ) ) ;
	else
                setStatus ( false,tr ( "Ok" ) );
}

void  PrgWid::verifyPic()
{
	if ( ! verifyDeviceType() )
		return ;

	QString qtxt="" ;

	emit messageGenerated ( qtxt ) ;

        setStatus ( true,tr ( "Verifying ..." ) ) ;
        QApplication::processEvents ( QEventLoop::ExcludeUserInputEvents , 200 ) ;
        ostringstream out ;

	int i = curpic->verify_pic ( out ) ;

	if ( i != 0 )
	{
		QString txt ( out.str().c_str() ) ;
		emit messageGenerated ( txt ) ;
		setStatus ( false,curpic->error_msg ( i ) ) ;
	}
	else
                setStatus ( false, tr ( "Ok" ) );
}

void  PrgWid::programPic()
{
	if ( ! verifyDeviceType() )
		return ;

	QString qtxt ;
        setStatus ( true, QString ( tr ( "Programming ..." ) ) ) ;
	emit messageGenerated ( qtxt ) ;

        QApplication::processEvents ( QEventLoop::ExcludeUserInputEvents , 400 ) ;

	ostringstream out ;
	int i = curpic->program_pic ( out ) ;

	if ( i != 0 )
	{
		qtxt = out.str().c_str() ;
		emit messageGenerated ( qtxt ) ;
		setStatus ( false,  curpic->error_msg ( i ) ) ;
	}
	else
                setStatus ( false, tr ( "Ok" ) );

}

void PrgWid::setStatus ( bool active, const QString& txt )
{
        progRunning->setOn(active) ;

	statusLabel->setText ( " "+txt+" " ) ;
        QApplication::processEvents ( QEventLoop::ExcludeUserInputEvents , 200 ) ;
}

//
// change filename - update UI
//
void PrgWid::setFileName ( const QString &fname )
{
	fileName = fname ;
        QString fn ( ( fname == "" ) ? tr ( "<none>" ) : fname )   ;
	fileNameLabel->setText ( fn ) ;
}

//
//	Find the KAction "name" in the KMainWindow actionCollection()
//
QAction * PrgWid::act ( const char *name ) const
{
        return actions.act(name) ;
}


/** No descriptions */
void PrgWid::slotDetectDeviceType()
{
	verifyDeviceType()  ;
}


/** Verify the currently plugged device type
\return true if detected device type is compatible with current pic model
or if device type is not autodetectable
 */
bool PrgWid::verifyDeviceType()
{
	QString dev  ;

	revdetected->setText ( "" ) ;

	if ( !curpic->isDetectable() )
	{
                detected->setText ( tr ( "<Non autodetectable device>" ) ) ;
		return true ;
	}

	dev = curpic->read_device_name().c_str()   ;

	if ( dev == "" )
	{
                detected->setText ( tr ( "<No device detected>" ) ) ;
		return false ;
	}
	else if ( dev == "?" )
	{
                detected->setText ( tr ( "<Unknown device>" ) ) ;
		return false ;
	}
	detected->setText ( dev ) ;
	revdetected->setText ( curpic->read_device_revision().c_str() ) ;

	bool ok =   verifyDeviceType ( dev ) ;
        setStatus ( false, ok ? tr ( "Ok" ) : tr ( "Mismatch" ) ) ;
	return ok ;
}

bool PrgWid::verifyDeviceType ( const QString& type )
{
	if ( type == curpic->specificName() )
		return true  ;

	// Not valid, ALERT
        int resp = QMessageBox::warning ( this,tr ( "Sorry" ),
                                          tr ( "Detected device type does not match specified target device." ),
                                          tr ( "Continue" ), tr ( "Abort" ) ) ;
	return resp == 0  ;
}
/** No descriptions */
bool PrgWid::queryClose()
{
	return true ;
}
/** No descriptions */
QStringList PrgWid::parseProgOptions ( const QString &op )
{
	QRegExp r ( "\\s*\\{([^}]+)\\}" ) ;
	int index = 0 ;
	QStringList list ;

	while ( r.search ( op, index ) != -1 )
	{
		index += r.matchedLength() ;
		list += r.cap ( 1 ) ;
	}
	return list ;
}

/** No descriptions */
void PrgWid::slotProgOptionSelected ( int opt )
{
	curpic->setProgOptionNumber ( opt ) ;
}
/** No descriptions */
void PrgWid::updateProgressBar ( int percent )
{
        progress->setValue ( percent );
        QApplication::processEvents ( QEventLoop::ExcludeUserInputEvents , 200 ) ;
}
/** No descriptions */
void PrgWid::updateProgressText ( const char * text )
{
	statusLabel->setText ( text );
        QApplication::processEvents ( QEventLoop::ExcludeUserInputEvents , 200 ) ;
}


/*!
    \fn PrgWid::slotExtraMenuManagement()
 */
void PrgWid::slotExtraMenuManagement()
{
	QString extra = programmer->extraFct() ;
	QStringList extraList = parseProgOptions ( extra ) ;

	QStringList::iterator i ;

        QMenu *pop = dynamic_cast<QMenu*>(sender()) ;

	pop->clear ();

	int nb ;
	for ( i = extraList.begin() , nb = 0 ; i != extraList.end() ; ++i, ++nb )
	{
		pop->insertItem ( *i , this, SLOT ( slotExecExtra ( int ) ), 0  , nb );
	}
}


/*!
    \fn PrgWid::slotExecExtra(int function_number)
 */
void PrgWid::slotExecExtra ( int function_number )
{
	programmer->executeExtraFct ( function_number ) ;
}
