#ifndef PRGWID_H
#define PRGWID_H

#include <QMainWindow>

#include <QProgressBar>

#include <QLabel>
#include <QComboBox>

#include "pic.hh"
#include "hardware.hh"
#include "progressListener.hh"
#include "ProgrammerBase.h"
#include "Indicator.h"
#include "ActionsCollection.h"

class PrgWid : public QMainWindow, public progressListener
{
  Q_OBJECT
  QString     fileName ;
  pic 	*curpic ;
  ProgrammerBase *programmer ;
  ActionsCollection actions ;


  // indicateur d'activite du programmeur
  Indicator *progRunning ;
  QLabel *statusLabel ;

  // status Bar
  QStatusBar * statBar ;

  // progress bar
  QProgressBar *progress ;

  QLineEdit *fileNameLabel ;
  QLineEdit * detected , *revdetected ;

public:
  PrgWid(ProgrammerBase *pr, const QString& fn, QWidget *parent) ;
  virtual ~PrgWid() ;
  void go() ;
  /** Verify the currently plugged device type
  \return true if detected device type is compatible with current pic model
  or if device type is not autodetectable
  */
  bool verifyDeviceType(const QString& type);
  bool verifyDeviceType();
  /** Called before the widget is closed */
  virtual bool queryClose();
  /** No descriptions */
  QStringList parseProgOptions(const QString &op);
  /** specified by progressObserver interface*/
  virtual void updateProgressBar(int percent);
  virtual void updateProgressText(const char *);
private:
  void setStatus(bool active, const QString& txt) ;
  void setFileName(const QString &fname) ;

public slots:
  void quit() ;
  void saveHex() ;
  void saveHexAs() ;
  void loadHex() ;
  void loadHexAs() ;

  void readPic() ;
  void erasePic() ;
  void verifyPic() ;
  void programPic() ;
  /** No descriptions */
  void slotDetectDeviceType();
  /** No descriptions */
  void slotProgOptionSelected(int opt);
    void slotExtraMenuManagement();
    void slotExecExtra(int function_number);

signals:
  void fileUpdated(const QString &) ;
  void bufferModified() ;
  void messageGenerated(QString&) ;
protected:
  /** Find a QAction "a" from the parent's actionCollection() */
  QAction * act(const char *a) const;
} ;
#endif
