/***************************************************************************
                          ProjectEditorWid.cpp  -  description
                             -------------------
    begin                : lun dc 8 2003
    copyright            : (C) 2003 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ProjectEditorWid.h"
#include "AsmConfigWidget.h"
#include "PikMain.h"

#include <qlayout.h>
#include <q3grid.h>
#include <q3vbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <q3textedit.h>
#include <q3groupbox.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <q3frame.h>
//Added by qt3to4:
#include <Q3GridLayout>
#include <QIcon>
#include <QFileDialog>

#include <cassert>

#include <iostream>
using namespace std ;

ProjectEditorWid::ProjectEditorWid ( Project *proj, QWidget *parent, const char *wname )
    : Q3MainWindow ( parent,wname, Qt::WDestructiveClose ), project ( proj )
{
    QIcon iconset(":/src/icons/22/pikdev_openfile.png") ;

    main_widget = PikMain::mainWindow() ;

    QStringList procs =  main_widget->getProcList() ;
    QStringList hfmt ;
    ( ( ( hfmt += "inhx8m" ) += "inhx8s" ) += "inhx16" ) += "inhx32" ;
    QStringList rad ;
    ( rad += "dec" ) += "hex" ;
    QStringList warn ;
    ( ( warn +="0" ) += "1" ) += "2" ;

    setCaption ( tr ( "Project editor" ) ) ;

    Q3VBox *vb = new Q3VBox ( this ) ;
    vb->setSpacing ( 8 ) ;
    vb->setLineWidth ( 2 );
    vb->setMargin ( 8 ) ;
    setCentralWidget ( vb ) ;


    Q3Grid *group = new Q3Grid ( 4,vb ) ;
    group->setSpacing ( 10 ) ;


    new QLabel ( tr ( "Project" ), group );
    QLineEdit *name = new QLineEdit ( group );
    name->setText ( project->projectName( ) ) ;
    name->setReadOnly ( true ) ;
    new QLabel ( tr ( "Type" ), group );
    QLineEdit *typname = new QLineEdit ( group );
    typname->setMaximumWidth ( 55 ) ;
    typname->setReadOnly ( true ) ;
    typname->setText ( project->getType( ) ) ;


    new QLabel ( tr ( "Directory" ), group );
    QLineEdit *dir = new QLineEdit ( group );
    dir->setText ( proj->projectPath( ) ) ;
    dir->setReadOnly ( true ) ;

    // Main grid

    Q3Frame *grid4x4 =  new Q3Frame ( vb ) ;
    Q3GridLayout *grid4x4L = new Q3GridLayout ( grid4x4 ,2,2,8,8 ) ;

    // -------------------------- global section ----------------------
    Q3GroupBox *globalbox = new Q3GroupBox ( 2,Qt::Horizontal,tr ( "Global" ),grid4x4 ) ;
    grid4x4L->addWidget ( globalbox,0,0 ) ;

    new QLabel ( tr ( "Description" ), globalbox ) ;
    glob_description=new Q3TextEdit ( globalbox ) ;
    glob_description->setText ( project->getDescription() ) ;
    new QLabel ( tr ( "Version" ), globalbox ) ;
    glob_version = new QLineEdit ( globalbox ) ;
    glob_version->setText ( project->getVersion() ) ;


    // -------------------------- asm params  section ----------------------
    Q3GroupBox *asmbox = new Q3GroupBox ( 2,Qt::Horizontal,tr ( "Assembler" ),grid4x4 ) ;
    grid4x4L->addWidget ( asmbox,0,1 ) ;
    //   Processor
    new QLabel ( tr ( "Processor" ), asmbox ) ;
    asm_device = new QComboBox ( asmbox ) ;
    asm_device->insertStringList ( procs );
    int k ;
    QString p=project->getDevice() ;
    if ( ( ( k=procs.findIndex ( p ) ) == -1 ) && ( k=procs.findIndex ( p+" (P)" ) ) == -1 )
        k = 0 ;
    asm_device->setCurrentItem ( k ) ;
    QToolTip::add
            ( asm_device,tr ( "PiKdev can only program processors marked with (P).\n"
                              "For the development of new programming drivers, please see\n"
                              "http://pikdev.free.fr, or contact the author." ) ) ;


    //   radix
    new QLabel ( tr ( "Radix" ),asmbox ) ;
    asm_radix = new QComboBox ( asmbox ) ;
    QToolTip::add
            ( asm_radix,tr ( "Choose the default radix used by gpasm to parse integer constants\n"
                             "* means that radix is specified by LIST directive." ) );
    asm_radix->insertItem ( "*" ) ;
    asm_radix->insertStringList ( rad );
    QString r = project->getRadix() ;
    k=rad.findIndex ( r ) ;
    asm_radix->setCurrentItem ( k+1 ) ;


    //  Warning level
    new QLabel ( tr ( "Warning level" ),asmbox ) ;
    asm_warning = new QComboBox ( asmbox ) ;
    asm_warning->insertItem ( "*" ) ;
    asm_warning->insertStringList ( warn );
    QString w=project->getWarnLevel() ;
    k=warn.findIndex ( w ) ;
    asm_warning->setCurrentItem ( k+1 ) ;
    QToolTip::add
            ( asm_warning,tr ( "For cleaner code, please choose '0'.\n"
                               "* means that warning level is specified by LIST directive." ) ) ;

    // list option
    new QLabel ( tr ( "List file" ),asmbox ) ;
    asm_list = new QCheckBox ( asmbox ) ;
    asm_list->setChecked ( project->getAsmList() ) ;


    //  Include Directories
    new QLabel ( tr ( "Include directory" ),asmbox ) ;
    Q3HBox *h1 = new Q3HBox ( asmbox ) ;
    asm_include = new QLineEdit ( h1 ) ;
    QString i=project->getIncludeDir() ;
    asm_include->setText ( i ) ;
    QToolTip::add
            ( asm_include,tr ( "By using the pseudovariable $(PRJPATH), you can specify an include directory\n"
                               "relative to the project directory."	) ) ;
    QPushButton *incdir_button = new  QPushButton ( iconset,"", h1 ) ;
    connect ( incdir_button,SIGNAL ( clicked() ), this, SLOT ( slotSelectIncDir() ) ) ;


    //  other option
    new QLabel ( tr ( "Other options" ),asmbox ) ;
    asm_other = new QLineEdit ( asmbox ) ;
    QString o=project->getOtherOptions() ;
    asm_other->setText ( o ) ;
    QToolTip::add
            ( asm_other,tr ( "This item allows you to manually specify a gpasm \"exotic\" option.\n"
                             "The $(PRJPATH) pseudo variable if supported in this item as it is "
                             "in the \"Include Directories\" one." ) ) ;
    if ( project->isCproject() )
    {
        // ------------------------------- C section -----------------
        Q3GroupBox *Cbox = new Q3GroupBox ( 2,Qt::Horizontal,tr ( "C compiler" ),grid4x4 ) ;
        grid4x4L->addMultiCellWidget ( Cbox,1,1,0,1 ) ;

        new QLabel ( tr ( "Compiler" ),Cbox ) ;
        C_compiler = new QComboBox ( false,Cbox ) ;
        C_compiler->insertItem ( "cpik" ) ;

        // find correct entry and display it
        QString the_compiler ( project->getCCompiler() ) ;
        for ( k=0 ; k < C_compiler->count() ; ++k )
            if ( C_compiler->text ( k ) == the_compiler )
                break ;
        if ( k == C_compiler->count() )
            k = 0 ; // not found , take first entry
        C_compiler->setCurrentItem ( k ) ;
        QToolTip::add
                ( C_compiler,tr ( "Sorry, PiKdev currently supports only one compiler." ) ) ;

        //  Include Directories
        new QLabel ( tr ( "Include directory" ),Cbox ) ;
        Q3HBox *Ch1 = new Q3HBox ( Cbox ) ;
        C_include = new QLineEdit ( Ch1 ) ;
        C_include->setText ( project->getCIncludeDir() ) ;
        QToolTip::add
                ( C_include,tr ( "By using the pseudovariable $(PRJPATH), you can specify a directory\n"
                                 "relative to the project directory. Comma-separated list of directories allowed."	) ) ;
        QPushButton *Cincdir_button = new  QPushButton ( iconset,"", Ch1,"C include button" ) ;
        connect ( Cincdir_button,SIGNAL ( clicked() ), this, SLOT ( slotSelectCIncDir() ) ) ;


        //  Libs Directories
        new QLabel ( tr ( "Libs directory" ),Cbox ) ;
        Q3HBox *Ch2 = new Q3HBox ( Cbox ) ;
        C_libs = new QLineEdit ( Ch2 ) ;
        C_libs->setText ( project->getCLibsDir() ) ;
        QToolTip::add
                ( C_libs,tr ( "By using the pseudovariable $(PRJPATH), you can specify a directory\n"
                              "relative to the project directory. Comma-separated list of directories allowed."	) ) ;
        QPushButton *Clibs_button = new  QPushButton ( iconset,"", Ch2,"C libs button" ) ;
        connect ( Clibs_button,SIGNAL ( clicked() ), this, SLOT ( slotSelectCLibDir() ) ) ;

        // other options
        new QLabel ( tr ( "Other options" ),Cbox ) ;
        C_other = new QLineEdit ( Cbox ) ;
        C_other->setText ( project->getOtherCOptions() ) ;
        QToolTip::add
                ( C_other,tr ( "This item allows you to manually specify a C \"exotic\" option.\n"
                               "The $(PRJPATH) pseudo variable if supported in this item as it is "
                               "in the \"Libraries Directory\" one." ) ) ;


        new QLabel ( tr ( "Optimize jumps" ),Cbox ) ;
        C_jmpopt = new QCheckBox ( Cbox ) ;
        C_jmpopt->setChecked ( project->getCJmpOpt() ) ;

        new QLabel ( "<hr>", Cbox ) ;
        new QLabel ( "<hr>", Cbox ) ;

        // display slb files option
        new QLabel ( tr ( "Display" ),Cbox ) ;
        Q3HBox *cbh1 = new Q3HBox ( Cbox ) ;

        new QLabel ( tr ( "Slb files" ), cbh1 ) ;
        C_slb = new QCheckBox ( cbh1 ) ;
        C_slb->setChecked ( project->getCSlb() ) ;

        // display asm files option
        new QLabel ( tr ( "Asm file" ), cbh1 ) ;
        C_asm = new QCheckBox ( cbh1 ) ;
        C_asm->setChecked ( project->getCAsm() ) ;


    }
    else
    {
        // ------------------------------- linker section -----------------
        Q3GroupBox *linkbox = new Q3GroupBox ( 2,Qt::Horizontal,tr ( "Linker" ),grid4x4 ) ;
        grid4x4L->addMultiCellWidget ( linkbox,1,1, 0,1 ) ;
        // hex file format
        new QLabel ( tr ( "Hex file format" ),linkbox ) ;
        link_fformat = new QComboBox ( linkbox ) ;
        link_fformat->insertStringList ( hfmt );
        QString h=project->getHexFormat() ;
        if ( ( k=hfmt.findIndex ( h ) ) == -1 )
            k = 0 ;
        ;
        link_fformat->setCurrentItem ( k ) ;
        QToolTip::add
                ( link_fformat,tr ( "CAUTION: inhx8s format is currently not supported by the PiKdev programmer.\n"
                                    "If you need this support, please contact the author." ) ) ;

        // debug option
        new QLabel ( tr ( "Debug messages" ),linkbox ) ;
        link_debug = new QCheckBox ( linkbox ) ;
        link_debug->setChecked ( project->getLinkDebug() ) ;

        // map option
        new QLabel ( tr ( "Map file" ),linkbox ) ;
        link_map = new QCheckBox ( linkbox ) ;
        link_map->setChecked ( project->getLinkMap() ) ;

        //  object/libs Directories
        new QLabel ( tr ( "Libraries directory" ),linkbox ) ;
        Q3HBox *hb2 = new Q3HBox ( linkbox ) ;
        link_include = new QLineEdit ( hb2 ) ;
        link_include->setText ( project->getLibsDir() ) ;
        QToolTip::add
                ( link_include,tr ( "By using the pseudovariable $(PRJPATH), you can specify a directory\n"
                                    "relative to the project directory:" ) ) ;
        QPushButton *libdir_button = new  QPushButton ( iconset,"", hb2 ) ;
        connect ( libdir_button,SIGNAL ( clicked() ), this, SLOT ( slotSelectLibDir() ) ) ;

        // other options
        new QLabel ( tr ( "Other options" ),linkbox ) ;
        link_other = new QLineEdit ( linkbox ) ;
        link_other->setText ( project->getOtherLinkOptions() ) ;
        QToolTip::add
                ( link_other,tr ( "This item allows you to manually specify a gplink \"exotic\" option.\n"
                                  "The $(PRJPATH) pseudo variable if supported in this item as it is "
                                  "in the \"Libraries Directory\" one." ) ) ;
    }
    // ------------------------- Buttons section ----------------------------

    Q3HBox *hbox7 = new Q3HBox ( vb ) ;
    hbox7->setSpacing ( 20 ) ;

    Q3HBox *space = new Q3HBox ( hbox7 ) ;
    QPushButton *bok = new QPushButton ( tr ( "Ok" ),hbox7 ) ;
    QPushButton *bapply = new QPushButton ( tr ( "Apply" ),hbox7 ) ;
    QPushButton *bcancel = new QPushButton ( tr ( "Cancel" ),hbox7 ) ;
    hbox7->setStretchFactor ( space,1 ) ;
    hbox7->setStretchFactor ( bcancel,0 ) ;
    hbox7->setStretchFactor ( bapply,0 ) ;
    hbox7->setStretchFactor ( bok,0 ) ;

    // Connect buttons
    connect ( bcancel, SIGNAL ( clicked() ), this, SLOT ( slotCancel() ) ) ;
    connect ( bapply, SIGNAL ( clicked() ), this, SLOT ( slotApply() ) ) ;
    connect ( bok, SIGNAL ( clicked() ), this, SLOT ( slotOk() ) ) ;

#if 0
    // disable link or C box, depending on type of project.
    if ( project->isCproject() )
        linkbox->setEnabled ( false ) ;
    else
        Cbox->setEnabled ( false ) ;
#endif

     setWindowIcon(QPixmap(":/src/icons/16/pikdev_icon.png") ) ;

    show() ;
}
ProjectEditorWid::~ProjectEditorWid()
{}
/** No descriptions */
void ProjectEditorWid::slotCancel()
{
    close ( true ) ;
}
/** No descriptions */
void ProjectEditorWid::slotApply()
{
    project->setDescription ( glob_description->text() ) ;
    project->setVersion ( glob_version->text() ) ;

    QString dev =  asm_device->currentText()   ;
    // remove the (P) suffix if it exists
    if ( dev.right ( 4 ) == " (P)" )
        dev.remove ( dev.length()-4, 4 ) ;
    project->setDevice ( dev ) ;
    project->setWarnLevel ( asm_warning->currentText() ) ;
    project->setOtherOptions ( asm_other->text() ) ;
    project->setIncludeDir ( asm_include->text() ) ;
    project->setRadix ( asm_radix->currentText() ) ;
    project->setAsmList ( asm_list->isOn() ) ;

    if ( project->isCproject() )
    {
        project->setCLibsDir ( C_libs->text() ) ;
        project->setOtherCOptions ( C_other->text() ) ;
        project->setCIncludeDir ( C_include->text() ) ;
        project->setCCompiler ( C_compiler->currentText() ) ;
        project->setCSlb ( C_slb->isOn() ) ;
        project->setCJmpOpt ( C_jmpopt->isOn() ) ;
        project->setCAsm ( C_asm ->isOn() ) ;
    }
    else
    {
        project->setHexFormat ( link_fformat->currentText() ) ;
        project->setLinkDebug ( link_debug->isOn() ) ;
        project->setLinkMap ( link_map->isOn() ) ;
        project->setLibsDir ( link_include->text() ) ;
        project->setOtherLinkOptions ( link_other->text() ) ;
    }

    project->commit() ;
}
/** No descriptions */
void ProjectEditorWid::slotOk()
{
    slotApply();
    slotCancel() ;
}
/** No descriptions */
void ProjectEditorWid::slotSelectIncDir()
{
    QString selected ;  // not used
    QString dir = QFileDialog::getOpenFileName(this, tr("Select include directory"), ".", "*",
                                               &selected, QFileDialog::ShowDirsOnly) ;

    if ( !dir.isEmpty() )
    {
        asm_include->setText ( dir ) ;
    }
}

/** No descriptions */
void ProjectEditorWid::slotSelectLibDir()
{
    QString selected ;  // not used
    QString dir = QFileDialog::getOpenFileName(this, tr("Select libraries directory"), ".", "*",
                                               &selected, QFileDialog::ShowDirsOnly) ;

    if ( !dir.isEmpty() )
    {
        link_include->setText ( dir ) ;
    }
}
/** No descriptions */
void ProjectEditorWid::slotSelectCIncDir()
{
    QString selected ;  // not used
    QString dir = QFileDialog::getOpenFileName(this, tr("Select include directory"), ".", "*",
                                               &selected, QFileDialog::ShowDirsOnly) ;

    if ( !dir.isEmpty() )
    {
        QString t = C_include->text() ;
        if ( t == "" )
            C_include->setText ( dir ) ;
        else
            C_include->setText ( t+","+dir ) ;
    }

}
/** No descriptions */
void ProjectEditorWid::slotSelectCLibDir()
{
    QString selected ;  // not used
    QString dir = QFileDialog::getOpenFileName(this, tr("Select libraries directory"), ".", "*",
                                               &selected, QFileDialog::ShowDirsOnly) ;
    if ( !dir.isEmpty() )
    {
        QString t = C_libs->text() ;
        if ( t == "" )
            C_libs->setText ( dir ) ;
        else
            C_libs->setText ( t+","+dir ) ;
    }

}
