/***************************************************************************
                          ProjectWid.h  -  description
                             -------------------
    begin                : sam jui 5 2003
    copyright            : (C) 2003 by Alain Gibaud
    email                : alain.gibaud@univ-valenciennes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROJECTWID_H
#define PROJECTWID_H

#include <qwidget.h>
#include <q3vbox.h>
#include <q3listview.h>
#include <qtimer.h>
//Added by qt3to4:
#include <QDragEnterEvent>
#include <QDropEvent>
#include <Q3PopupMenu>

#include "Project.h"
#include "PikMain.h"

/**A widget to display and edit  informations about the current project
  *@author Alain Gibaud
  */
class NewProject ;
class Q3PopupMenu ;
class PikMain ;
class ProjectEditorWid ;
class ConfigBitsWizard ;

class ProjectWid : public Q3VBox
{
		Q_OBJECT
	public:
		ProjectWid ( PikMain *mainwindow, QWidget *parent, const char *name );
		~ProjectWid();
		/** Add a file name to the project tree
		(filename is absolute or relative to the project file location) */
		void addFile2Tree ( const QString&name );
		/** Returns the current project object (might be NULL) */
		Project *getProject();
		/** return the main window */
		PikMain *mainWindow() ;
		/** Drag and drop management
		*/
		virtual void dragEnterEvent ( QDragEnterEvent * );
		/** Drop on project widget */
		virtual void dropEvent ( QDropEvent *ev );

                void updateCEntryPoints ( QBEditorDocument *doc , Q3ListViewItem *it );
                void updateAsmEntryPoints ( QBEditorDocument *doc, Q3ListViewItem *it );

	public slots:
		void slotProjectRButton ( Q3ListViewItem *, const QPoint &, int )   ;
		/** No descriptions */
		void slotCreateProject ( const QString&, const QString&, const QString& );
		/** No descriptions */
		void slotNewProjectClosed();
		/** No descriptions */
		void slotCloseProject();
		/** No descriptions */
		void slotInsertFile();
		/** No descriptions */
		void slotOpenProject();
		/** remove the current item from tree (and the file from project)  */
		void slotRemoveFile();
		/** No descriptions */
		void slotInsertCurrentFile();
		/** Calls the Project Editor Widget */
		void slotEditProject();
		/** No descriptions */
		void slotNewProject();
		/** Updates the UI when a project is opened/closed */
		void slotProjectIsOpen ( bool open );
		/** No descriptions */
		void slotProjectEditorClosed();
		/** No descriptions */
		void slotOpenProjectFile ( const QString& filename ) ;
		virtual void slotUpdateEntryPoints();
		/** activate config bits editor */
    		void slotEditConfigBits();
		/** do housekeeping when cf bits editor is closed */
    		void slotConfigBitsEditorClosed();
	private: // Private attributes
		/** New project widget */
		NewProject * newproject;
		/** Last selected item */
		Q3ListViewItem *currentListViewItem ;
		/** Project list view */
		Q3ListView *projView ;
		/** Root of Project tree */
		Q3ListViewItem *projectTree;
		Project *project ;
		/** Link to main window */
		PikMain *main_window ;
		/** Project editor, non null when open */
		ProjectEditorWid * projecteditor;
		/** Config bits editor, non null when open */
		ConfigBitsWizard *configbits_editor ;
		/** Timer for triggering of source code parsing */
		QTimer *entryPointTimer ;
	private: // Private methods
		/** Make an empty popup menu with "title" header */
                QMenu *makePopup ( const QString& title, QWidget  *parent );
		/** insert a new absolute filename - returns true if OK, false if already exists */
		bool insertAbsFile ( const QString &fname ) ;
		/** remove all children of list */
		void removeChildren ( Q3ListViewItem *list ) ;
                /** open the document  which corresponds to this item -
                        returns adress of the editor, or 0 if doc is not found  */
                QBEditor * openEditor ( Q3ListViewItem *item ) ;

	private slots: // Private slots
		/** No descriptions */
		void slotProjectElementClicked ( Q3ListViewItem * );
	public: // Public attributes
};

#endif
